@@ -44,7 +44,6 @@ import re
 import textwrap
 from typing import List, Iterator, Tuple, Optional
 
-from io import StringIO
 from debian.changelog import (
     ChangeBlock,
     Changelog,
@@ -68,17 +67,24 @@ class ChangelogEditor(Editor):
 
     def __init__(
             self, path: str = 'debian/changelog',
-            allow_reformatting: Optional[bool] = False):
+            allow_reformatting: Optional[bool] = False,
+            allow_missing: bool = False):
         super(ChangelogEditor, self).__init__(
             path, allow_reformatting=allow_reformatting)
+        self.allow_missing = allow_missing
+
+    @classmethod
+    def create(cls, path: str = 'debian/changelog'):
+        return cls(path, allow_reformatting=True, allow_missing=True)
 
     def _parse(self, content):
-        return Changelog(content)
+        cl = Changelog()
+        cl.parse_changelog(
+            content, max_blocks=None, allow_empty_author=True, strict=False)
+        return cl
 
     def _format(self, parsed):
-        f = StringIO()
-        parsed.write_to_open_file(f)
-        return f.getvalue()
+        return parsed._format(allow_missing_author=True)
 
     @property
     def changelog(self):
@@ -87,9 +93,18 @@ class ChangelogEditor(Editor):
     def __getitem__(self, i):
         return self.changelog[i]
 
+    def _nonexistant(self):
+        if self.allow_missing:
+            return Changelog()
+        raise
+
     def new_block(self, *args, **kwargs):
         return self.changelog.new_block(*args, **kwargs)
 
+    def auto_version(self, version, **kwargs):
+        return changelog_auto_version(
+            self.changelog, version=version, **kwargs)
+
     def add_entry(
             self,
             summary: List[str],
@@ -101,6 +116,38 @@ class ChangelogEditor(Editor):
             timestamp=timestamp, urgency=urgency)
 
 
+def changelog_auto_version(
+        cl: Changelog, version: Version,
+        maintainer: Optional[Tuple[str, str]] = None,
+        timestamp: Optional[datetime] = None,
+        package: Optional[str] = None,
+        urgency: str = 'low'):
+    """Update current changelog entry to version or create a new one.
+    """
+    if maintainer is None:
+        maintainer_name, maintainer_email = get_maintainer()
+    else:
+        maintainer_name, maintainer_email = maintainer
+    if maintainer_name is None or maintainer_email is None:
+        raise ValueError(
+            'unable to determine maintainer details and none specified')
+    if package is None:
+        package = cl[0].package
+    if timestamp is None:
+        timestamp = datetime.now()
+    if len(cl) > 0 and distribution_is_unreleased(cl[0].distributions):
+        cl[0].version = version
+        cl[0].date = format_datetime(timestamp)
+        cl[0].package = package
+        # TODO(jelmer): Also set maintainer again?
+    else:
+        cl.new_block(
+            version=version, package=package,
+            distributions='UNRELEASED', urgency=urgency,
+            author="%s <%s>" % (maintainer_name, maintainer_email),
+            date=format_datetime(timestamp))
+
+
 def changes_sections(
         changes: List[str]
         ) -> Iterator[
@@ -308,8 +355,8 @@ def changelog_add_entry(
     if maintainer_name is None or maintainer_email is None:
         raise ValueError(
             'unable to determine maintainer details and none specified')
-    if (cl[0].distributions == 'UNRELEASED' or (
-            cl[0].author is None and cl[0].date is None)):
+    if (distribution_is_unreleased(cl[0].distributions) or (
+                cl[0].author is None and cl[0].date is None)):
         by_author = list(changes_by_author(cl[0].changes()))
         if all([author is None for (author, linenos, change) in by_author]):
             if cl[0].author is not None:
@@ -324,7 +371,7 @@ def changelog_add_entry(
                 if cl[0]._changes[-1]:
                     cl[0]._changes.append('')
                 cl[0]._changes.append('  [ %s ]' % maintainer_name)
-        if not cl[0]._changes[-1].strip():
+        if cl[0] ._changes and not cl[0]._changes[-1].strip():
             del cl[0]._changes[-1]
     else:
         cl.new_block(
@@ -436,6 +483,12 @@ def all_sha_prefixed(cb: ChangeBlock) -> bool:
     return (sha_prefixed > 0)
 
 
+def distribution_is_unreleased(distribution):
+    return (
+        distribution == 'UNRELEASED' or
+        distribution.startswith('UNRELEASED-'))
+
+
 def changeblock_ensure_first_line(block, line, maintainer=None):
     """Ensure that the first line matches the specified line.
     """
@@ -463,7 +516,7 @@ def changeblock_ensure_first_line(block, line, maintainer=None):
 def release(cl, distribution="unstable", timestamp=None, localtime=True):
     """Create a release for a changelog file.
     """
-    if cl[0].distributions == 'UNRELEASED':
+    if distribution_is_unreleased(cl[0].distributions):
         cl[0].distributions = distribution
         cl[0].date = format_date(timestamp=timestamp, localtime=localtime)
 
@@ -478,7 +531,7 @@ def find_last_distribution(changelog):
     """
     for block in changelog._blocks:
         distribution = block.distributions.split(" ")[0]
-        if distribution != "UNRELEASED":
+        if not distribution_is_unreleased(distribution):
             return distribution
     return None
 
@@ -522,3 +575,22 @@ def find_thanks(changes):
             thanks_str = re.sub(r"\s+", " ", thanks_str)
             thanks.append(thanks_str)
     return thanks
+
+
+def upstream_merge_changelog_line(upstream_version):
+    """Describe that a new upstream revision was merged.
+
+    This will either describe that a new upstream release or a new upstream
+    snapshot was merged.
+
+    :param upstream_version: Upstream version string
+    :return: Line string for use in changelog
+    """
+    vcs_suffixes = ["~bzr", "+bzr", "~svn", "+svn", "~git", "+git", "-git"]
+    for vcs_suffix in vcs_suffixes:
+        if vcs_suffix in str(upstream_version):
+            entry_description = "New upstream snapshot."
+            break
+    else:
+        entry_description = "New upstream release."
+    return entry_description
