@@ -241,10 +241,16 @@ class ControlEditor(object):
     changed: bool
 
     def __init__(self, path: str = 'debian/control',
-                 allow_reformatting: Optional[bool] = None):
+                 allow_reformatting: Optional[bool] = None,
+                 allow_missing: bool = False):
         self.path = path
         self._primary = Deb822Editor(
-            path, allow_reformatting=allow_reformatting)
+            path, allow_reformatting=allow_reformatting,
+            allow_missing=allow_missing)
+
+    @classmethod
+    def create(cls, path: str = 'debian/control'):
+        return cls(path, allow_reformatting=True, allow_missing=True)
 
     @classmethod
     def from_tree(cls, tree, subpath=None):
@@ -261,9 +267,10 @@ class ControlEditor(object):
     @property
     def source(self) -> Deb822:
         """Source package."""
-        if not self._primary.paragraphs[0].get('Source'):
+        if (len(self.paragraphs[0]) > 0 and
+                not self.paragraphs[0].get('Source')):
             raise ValueError('first paragraph is not Source')
-        return self._primary.paragraphs[0]
+        return self.paragraphs[0]
 
     @property
     def binaries(self) -> List[Deb822]:
@@ -334,6 +341,13 @@ class ControlEditor(object):
             self.changed = self._primary.changed
         return False
 
+    def add_binary(self, contents):
+        if isinstance(contents, dict):
+            para = Deb822(contents)
+        else:
+            para = contents
+        return self._primary.paragraphs.append(para)
+
 
 def parse_relations(text: str):
     """Parse a package relations string.
@@ -752,6 +766,8 @@ def delete_from_list(liststr: str, item_to_delete: str) -> str:
 
 def is_dep_implied(dep: PkgRelation, outer: PkgRelation) -> bool:
     """Check if one dependency is implied by another.
+
+    Is dep implied by outer?
     """
     if dep.name != outer.name:
         return False
@@ -791,6 +807,15 @@ def is_dep_implied(dep: PkgRelation, outer: PkgRelation) -> bool:
             return False
         else:
             raise AssertionError('unsupported: %s' % outer.version[0])
+    elif dep.version[0] == '>>':
+        if outer.version[0] == '>>':
+            return Version(outer.version[1]) >= Version(dep.version[1])
+        elif outer.version[0] in ('=', '>='):
+            return Version(outer.version[1]) > Version(dep.version[1])
+        elif outer.version[0] in ('<<', '<='):
+            return False
+        else:
+            raise AssertionError('unsupported: %s' % outer.version[0])
     else:
         raise AssertionError('unable to handle checks for %s' % dep.version[0])
 
