@@ -99,6 +99,16 @@ def read_debhelper_compat_file(path: str) -> int:
         return int(line.strip())
 
 
+def get_debhelper_compat_level_from_control(control) -> Optional[int]:
+    try:
+        offset, [relation] = get_relation(
+            control.get("Build-Depends", ""), "debhelper-compat")
+    except (IndexError, KeyError):
+        return None
+    else:
+        return int(str(relation.version[1]))
+
+
 def get_debhelper_compat_level(path: str = '.') -> Optional[int]:
     try:
         return read_debhelper_compat_file(os.path.join(path, 'debian/compat'))
@@ -111,13 +121,7 @@ def get_debhelper_compat_level(path: str = '.') -> Optional[int]:
     except FileNotFoundError:
         return None
 
-    try:
-        offset, [relation] = get_relation(
-            control.get("Build-Depends", ""), "debhelper-compat")
-    except (IndexError, KeyError):
-        return None
-    else:
-        return int(str(relation.version[1]))
+    return get_debhelper_compat_level_from_control(control)
 
 
 @dataclass
@@ -222,7 +226,7 @@ class MaintscriptEditor(Editor):
         ret = []
         for line in content.splitlines(True):
             if line.startswith('#') or not line.strip():
-                ret.append(line)
+                ret.append(line.rstrip('\n'))
             else:
                 ret.append(parse_maintscript_line(line))
         return ret
@@ -238,11 +242,28 @@ class MaintscriptEditor(Editor):
         return [entry for entry in self.lines
                 if not isinstance(entry, str)]
 
+    def __delitem__(self, req):
+        ei = 0
+        # TODO(jelmer): Also remove preceding comments?
+        for i, e in enumerate(self.lines):
+            if not isinstance(e, str):
+                if ei == req:
+                    del self.lines[i]
+                    return
+                ei += 1
+        raise IndexError(req)
+
+    def __getitem__(self, req):
+        return self.entries[req]
+
+    def __len__(self):
+        return len(self.entries)
+
     def append(self, entry):
         if self._parsed is None:
-            self._parsed = [entry, '\n']
+            self._parsed = [entry]
         else:
-            self._parsed.extend([entry, '\n'])
+            self._parsed.extend([entry])
 
     def _format(self, parsed):
         """Serialize the parsed object."""
@@ -251,9 +272,9 @@ class MaintscriptEditor(Editor):
         ret = []
         for entry in self._parsed:
             if isinstance(entry, str):
-                ret.append(entry)
+                ret.append(entry + '\n')
             else:
-                ret.append(serialize_maintscript_line(entry.args()))
+                ret.append(serialize_maintscript_line(entry.args()) + '\n')
         if ret:
             return ''.join(ret)
         return None
