@@ -18,12 +18,12 @@
 """Functions for working with watch files."""
 
 from io import StringIO
+import logging
 import re
 import sys
 from typing import (
     Iterable, List, Union, Callable, Optional, TextIO, Iterator, Tuple)
 from urllib.parse import urljoin
-from warnings import warn
 
 from debian.changelog import Version
 
@@ -155,7 +155,7 @@ def parse_sed_expr(vm):
     raise InvalidUVersionMangle(vm, 'not a substitution or translation regex')
 
 
-def parse_subst_expr(vm):
+def parse_subst_expr(vm: str) -> Tuple[str, str, Optional[str]]:
     if vm[0] != 's':
         raise InvalidUVersionMangle(vm, 'not a substitution regex')
     parts = re.split(r'(?<!\\)' + vm[1], vm)
@@ -163,11 +163,15 @@ def parse_subst_expr(vm):
         raise InvalidUVersionMangle(vm)
     pattern = parts[1]
     replacement = parts[2]
-    flags = parts[3]
+    flags: Optional[str]
+    try:
+        flags = parts[3]
+    except IndexError:
+        flags = None
     return (pattern, replacement, flags)
 
 
-def parse_transl_expr(vm: str) -> Tuple[str, str, str]:
+def parse_transl_expr(vm: str) -> Tuple[str, str, Optional[str]]:
     if not vm.startswith('tr'):
         raise InvalidUVersionMangle(vm, 'not a translation regex')
     parts = re.split(r'(?<!\\)' + vm[2], vm)
@@ -175,7 +179,11 @@ def parse_transl_expr(vm: str) -> Tuple[str, str, str]:
         raise InvalidUVersionMangle(vm)
     pattern = parts[1]
     replacement = parts[2]
-    flags = parts[3]
+    flags: Optional[str]
+    try:
+        flags = parts[3]
+    except IndexError:
+        flags = None
     return (pattern, replacement, flags)
 
 
@@ -186,7 +194,7 @@ def apply_sed_expr(vm: str, orig: str) -> str:
         return re.sub(pattern, replacement.replace('$', '\\'), orig)
     elif kind == 'tr':
         from tr import tr
-        return tr(pattern, replacement, orig, flags)
+        return tr(pattern, replacement, orig, flags or '')
     else:
         raise ValueError(kind)
 
@@ -214,20 +222,22 @@ class Release(object):
             self.pgpsigurl)
 
 
-def html_search(body, matching_pattern):
+def html_search(body, matching_pattern, base_url):
     from bs4 import BeautifulSoup
     soup = BeautifulSoup(body, 'html.parser')
+    matching_pattern = urljoin(base_url.rstrip('/') + '/', matching_pattern)
     for a in soup.find_all('a'):
         href = a.attrs.get('href')
         if not href:
             continue
+        href = urljoin(base_url.rstrip('/') + '/', href)
         m = re.fullmatch(matching_pattern, href)
         if m:
             yield m
 
 
-def plain_search(body, matching_pattern):
-    return re.finditer(matching_pattern, body)
+def plain_search(body, matching_pattern, base_url):
+    return re.finditer(matching_pattern.encode(), body)
 
 
 searchers = {
@@ -254,7 +264,11 @@ class Watch(object):
             vm = self.get_option('uversionmangle')
         except KeyError:
             return version
-        return apply_sed_expr(vm, version)
+        try:
+            return apply_sed_expr(vm, version)
+        except re.error as e:
+            raise WatchSyntaxError(
+                'invalid uversionmangle %r: %s' % (vm, e)) from e
 
     def get_option(self, name):
         for option in self.options:
@@ -334,9 +348,11 @@ class Watch(object):
             searchmode = self.get_option('searchmode')
         except KeyError:
             searchmode = 'html'
+        logging.debug('Fetching url %s; searchmode=%s', url, searchmode)
         req = Request(url, headers={'User-Agent': user_agent})
         resp = urlopen(req)
-        for m in searchers[searchmode](resp.read(), self.matching_pattern):
+        for m in searchers[searchmode](
+                resp.read(), self.matching_pattern, url):
             # TODO(jelmer): Apply uversionmangle
             full_url = urljoin(url, m.group(0))
             try:
@@ -352,6 +368,10 @@ class MissingVersion(Exception):
     """The version= line is missing."""
 
 
+class WatchSyntaxError(Exception):
+    """Syntax error in watch file."""
+
+
 def parse_watch_file(f: Iterable[str]) -> Optional[WatchFile]:
     """Parse a watch file.
 
@@ -374,7 +394,7 @@ def parse_watch_file(f: Iterable[str]) -> Optional[WatchFile]:
             continued = []
     if continued:
         # Hmm, broken line?
-        warn('watchfile ended with \\; skipping last line')
+        logging.warning('watchfile ended with \\; skipping last line')
         lines.append(continued)
     if not lines:
         return None
@@ -424,7 +444,7 @@ def parse_watch_file(f: Iterable[str]) -> Optional[WatchFile]:
             m = re.findall(r'/([^/]*\([^/]*\)[^/]*)$', url)
             if m:
                 parts = [m[0]] + line.split(maxsplit=1)
-                url = url[:-len(m[0])-1]
+                url = url[:-len(m[0])-1].strip()
             else:
                 parts = line.split(maxsplit=2)
             entries.append(Watch(url, *parts, opts=opts))  # type: ignore
@@ -465,11 +485,20 @@ class WatchEditor(Editor):
 
 def uscan(wf, package):
     for entry in wf.entries:
+        logging.info('entry: %s' % entry)
         for d in entry.discover(package):
-            print('%s' % d)
+            logging.info('  %s' % d)
 
 
 def main(argv):
+    import argparse
+    parser = argparse.ArgumentParser()
+    parser.add_argument('--verbose', action='store_true')
+    args = parser.parse_args()
+    if args.verbose:
+        logging.basicConfig(level=logging.DEBUG, format='%(message)s')
+    else:
+        logging.basicConfig(level=logging.INFO, format='%(message)s')
     with open('debian/watch', 'r') as f:
         wf = parse_watch_file(f)
     from debian.deb822 import Deb822
